#!/usr/bin/env python
import json
import os
import re
import subprocess

import click
from click.exceptions import ClickException
from cookiecutter.main import cookiecutter

from arcl import __version__
from arcl.auth import (
    AUTH_ACTIONS,
    AUTH_ACTIONS_GET_TOKEN,
    AUTH_ACTIONS_LOGIN,
    AUTH_ACTIONS_LOGOUT,
    AUTH_ACTIONS_SHOW,
    ArchimedesAuth,
    get_accepted_env,
)
from arcl.config import get_config_path

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
COOKIECUTTER_TEMPLATE_GIT_URL = (
    "https://github.com/OptimeeringAS/archimedes-cookiecutter.git"
)
DEFAULT_AUTHOR_NAME = "Optimeering AS <dev@optimeering.com>"

# source for official regex for semver:
# https://semver.org/#is-there-a-suggested-regular-expression-regex-to-check-a-semver-string
SEMVER_REGEX = (
    r"(?P<major>0|[1-9]\d*)\.(?P<minor>0|[1-9]\d*)\.(?P<patch>0|[1-9]\d*)"
    r"(?:-(?P<prerelease>(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)"
    r"(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?"
    r"(?:\+(?P<buildmetadata>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"
)


class ConfigNotFoundException(Exception):
    """Exception to raise when config not in config path"""


def get_author_name(user):
    """Returns the authors name"""
    name = user.get("name")
    email = user.get("email")
    username = user.get("username")

    for author_name in [name, email]:
        if author_name:
            return author_name

    return username


def get_python_version():
    """Returns active python version"""
    errors = []

    for python_binary_name in ["python3", "python"]:
        try:
            python_version_output = subprocess.run(
                f"{python_binary_name} --version",
                shell=True,
                check=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.DEVNULL,
            )
            return _get_version_from_python_version_string(python_version_output)
        except subprocess.CalledProcessError as err:
            errors.append(err)

    print(
        "Error while trying to run `python --version`. "
        "Please make sure that python is installed "
        "and `python` is available in path."
    )
    print(errors)
    raise Exception("Error while trying to get available python version")


def _get_version_from_python_version_string(version_string):
    result = re.search(SEMVER_REGEX, version_string.stdout.decode("utf-8").strip())
    result_dict = result.groupdict()
    return f"{result_dict['major']}.{result_dict['minor']}.{result_dict['patch']}"


def get_config(project_name, env):
    """Create a config for the project"""
    author_name = DEFAULT_AUTHOR_NAME

    if os.path.exists(get_config_path(env)):
        with open(get_config_path(env), "r", encoding="utf8") as file:
            config = json.loads(file.read())
        author_name = get_author_name(config["user"])

    return {
        "project_name": project_name,
        "author_name": author_name,
        "python_version": get_python_version(),
    }


@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    """
    Welcome to The Archimedes CLI.

    \b
    Commands:
        arcl new         Create a new project
        arcl auth        Handle Archimedes authentication
        arcl version     Print the version number
    """


@cli.command(hidden=True)
@click.argument(
    "action", type=click.Choice(AUTH_ACTIONS, case_sensitive=False), required=True
)
@click.argument("organization", required=False)
@click.option(
    "--env",
    type=click.STRING,
    required=False,
    default="prod",
    help="Environment to setup",
)
def auth(action, organization, env):
    """Method for handling auth operations"""
    accepted_env = get_accepted_env()
    if env not in accepted_env:
        raise ClickException(f"env should be one of {accepted_env}")

    archimedes_auth = ArchimedesAuth(env)

    if action.lower() == AUTH_ACTIONS_LOGOUT:
        archimedes_auth.logout()
        return

    if action.lower() == AUTH_ACTIONS_LOGIN:
        archimedes_auth.login(organization)
        return

    if action.lower() == AUTH_ACTIONS_GET_TOKEN:
        click.echo(archimedes_auth.get_access_token())

    if action.lower() == AUTH_ACTIONS_SHOW:
        click.echo(archimedes_auth.show())


@cli.command(hidden=True)
@click.argument("name", required=True)
@click.option(
    "--template-version",
    "-t",
    "template_version",
    help=f"Version of the template to use. "
    f"It can be a branch, name or "
    f"tag of the git repo of "
    f"{COOKIECUTTER_TEMPLATE_GIT_URL}",
    default=None,
)
def new(name, template_version):
    """Creates a new arcl project"""
    try:
        project_dir = cookiecutter(
            COOKIECUTTER_TEMPLATE_GIT_URL,
            checkout=template_version,
            extra_context=get_config(name, "prod"),
            no_input=True,
        )
    except ConfigNotFoundException as err:
        click.echo(err)
        return

    click.echo("\nYour new project has been created!")
    click.echo("")
    click.echo(f'$ cd "{project_dir}"')
    click.echo("$ python -m pip install wheel pip --upgrade")
    click.echo("$ poetry update --lock")
    click.echo("$ poetry install")
    click.echo("")
    click.echo("to get started.")
    click.echo("")


@cli.command(hidden=True)
def version():
    """
    Print the current version
    """
    click.echo(__version__)


if __name__ == "__main__":
    cli()



import abc
import os

USER_HOME_DIR = os.path.expanduser("~")
ARCHIMEDES_CONF_DIR = os.path.join(USER_HOME_DIR, ".archimedes")

if not os.path.exists(ARCHIMEDES_CONF_DIR):
    os.mkdir(ARCHIMEDES_CONF_DIR)

ARCHIMEDES_API_CONFIG = {
    "prod": {
        "client_id": "5bc3a702-d753-43ff-9051-e7fdfdd95023",
        "aad_app_client_id": "c0a9f773-6276-4d71-8df6-7239e695aff6",
        "url": "https://api.fabapps.io",
        "authority": "https://login.microsoftonline.com/common",
    },
    "dev": {
        "client_id": "2e2f3c84-d1aa-49cc-90a2-fd3fa3380d27",
        "aad_app_client_id": "eaaa9f9f-395d-46aa-847f-b5fb6c087ff6",
        "url": "https://api-dev.fabapps.io",
        "authority": "https://login.microsoftonline.com/common",
    },
}


class InvalidEnvironmentException(Exception):
    """Exception class for invalid environment configuration"""


class ApiConfig(abc.ABC):  # pylint:disable=too-few-public-methods
    """Class for API configurations."""

    def __init__(self, env):
        self.config = ARCHIMEDES_API_CONFIG
        if env not in self.config:
            raise InvalidEnvironmentException(
                f"Invalid environment {env}, "
                f"supported values are "
                f"{', '.join([str(key) for key in self.config])}"
            )
        self.environment = env.lower()

    def __getattr__(self, item):
        env_config = self.config[self.environment]
        return env_config[item]


def get_api_config(environment):
    """Returns API config"""
    return ApiConfig(environment)


def get_config_path(env):
    """Returns config path."""
    return os.path.join(ARCHIMEDES_CONF_DIR, f"arcl-{env}.json")


def get_token_path(env):
    """Returns token path."""
    return os.path.join(ARCHIMEDES_CONF_DIR, f"msal-{env}.cache.bin")


def get_legacy_config_path():
    """Returns legacy config path."""
    return os.path.join(ARCHIMEDES_CONF_DIR, "arcl.json")


def get_legacy_token_path():
    """Returns legacy token path"""
    return os.path.join(ARCHIMEDES_CONF_DIR, "msal.cache.bin")



from msal_extensions import (
    FilePersistence,
    PersistedTokenCache,
    build_encrypted_persistence,
)

from arcl.config import get_token_path


def build_persistence(location, fallback_to_plaintext=False):
    """Build a suitable persistence instance based your current OS"""
    try:
        return build_encrypted_persistence(location)
    except:  # pylint:disable=bare-except
        if not fallback_to_plaintext:
            raise
        return FilePersistence(location)


def get_token_cache(env):
    """Get token from the cache"""
    persistence = build_persistence(get_token_path(env), True)
    token_cache = PersistedTokenCache(persistence)
    return token_cache



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['arcl']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'cookiecutter>=2.1.1,<3.0.0',
 'msal-extensions>=1.0.0,<2.0.0',
 'msal>=1.10.0,<2.0.0',
 'requests']

entry_points = \
{'console_scripts': ['arcl = arcl.cli:cli']}

setup_kwargs = {
    'name': 'arcl',
    'version': '1.7.6',
    'description': 'The CLI for Archimedes',
    'long_description': None,
    'author': 'Optimeering AS',
    'author_email': 'dev@optimeering.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<3.11',
}


setup(**setup_kwargs)
